MODULE Main;
(** =========================================================================  
   STM32F334 startup code (memory mapping control
   phase-locked loop setup etc.) 
  
   Target: STM32F334 microcontrollers
   
   (c) 2016-2017 CFB Software   
   http://www.astrobe.com  
 
   ========================================================================= *)

IMPORT 
  SYSTEM, MCU, Traps, In, Serial, Out, LinkOptions; (* must be last in the import list *)

CONST
  CR_HSION = 0;
  CR_HSIRDY = 1;
  CR_HSEON = 16;
  CR_HSERDY = 17;
  CR_HSEBYP = 18;
  CR_CSSON = 19;
  CR_PLLON = 24;
  CR_PLLRDY = 25;
  
  CSR_LSION = 0;
  CSR_LSIRDY = 1;
  
  CFGR_PLLSRC = 16;

    
PROCEDURE* ResetClocks;
VAR
  x: SET;
BEGIN
  (* switch off the high-speed internal clock *)
  SYSTEM.GET(MCU.RCC_CR, x);
  SYSTEM.PUT(MCU.RCC_CR, x - {CR_HSION});

  (* reset CFGR register *)
  SYSTEM.PUT(MCU.RCC_CFGR, 0);

  (* reset HSEON, CSSON and PLLON bits *)
  SYSTEM.GET(MCU.RCC_CR, x);
  SYSTEM.PUT(MCU.RCC_CR, x - {CR_HSEON, CR_CSSON, CR_PLLON});
  REPEAT UNTIL ~SYSTEM.BIT(MCU.RCC_CR, CR_PLLRDY);

  (* reset HSE bypass bit *)
  SYSTEM.GET(MCU.RCC_CR, x);
  SYSTEM.PUT(MCU.RCC_CR, x - {CR_HSEBYP});
  (* disable all interrupts *)
  SYSTEM.PUT(MCU.RCC_CIR, 0)
END ResetClocks;

PROCEDURE InitClocks;
VAR
  x: SET;
  SYSCLK: INTEGER;
  HCLK, PCLK1, PCLK2: INTEGER;
BEGIN
  (* PWR initialization *)
  SYSTEM.PUT(MCU.RCC_APB1ENR, {MCU.RCC_APB1ENR_PWR});
  
  (* setup internal clock and wait for HSI stabilization *)
  SYSTEM.GET(MCU.RCC_CR, x);
  SYSTEM.PUT(MCU.RCC_CR, x + {CR_HSION});
  REPEAT UNTIL SYSTEM.BIT(MCU.RCC_CR, CR_HSIRDY);

  (* configure high-speed external clock *)
  SYSTEM.GET(MCU.RCC_CR, x);
  x := x + {CR_HSEBYP}; 
  SYSTEM.PUT(MCU.RCC_CR, x);
  SYSTEM.PUT(MCU.RCC_CR, x + {CR_HSEON});
  REPEAT UNTIL SYSTEM.BIT(MCU.RCC_CR, CR_HSERDY); 
  
  (* activate PLL x 4 on HSE *)
  SYSTEM.GET(MCU.RCC_CFGR, x);
  SYSTEM.PUT(MCU.RCC_CFGR, x + {19} + {CFGR_PLLSRC} + {1, 0}); 
  SYSTEM.GET(MCU.RCC_CR, x);
  SYSTEM.PUT(MCU.RCC_CR, x + {CR_PLLON});
  REPEAT UNTIL SYSTEM.BIT(MCU.RCC_CR, CR_PLLRDY);
  
  (* configure low-speed clock *)
  SYSTEM.GET(MCU.RCC_CSR, x);
  SYSTEM.PUT(MCU.RCC_CSR, x + {CSR_LSION});
  REPEAT UNTIL SYSTEM.BIT(MCU.RCC_CSR, CSR_LSIRDY);

  (* configure flash *)
  SYSTEM.GET(MCU.FLASH_ACR, x);
  SYSTEM.PUT(MCU.FLASH_ACR, x + {4, 1}); (* Enable prefetch, 2 wait states *)
  
END InitClocks;

PROCEDURE SetupPLL;
BEGIN
  ResetClocks; 
  InitClocks 
END SetupPLL;

PROCEDURE* InitFPU;
CONST
  CPACR = 0E000ED88H;
VAR
  s: SET;
BEGIN
  (* Enable access to the Floating Point Coprocessor *)
  SYSTEM.GET(CPACR, s);
  SYSTEM.PUT(CPACR, s + {20..23});
  SYSTEM.EMIT(MCU.DSB);
  SYSTEM.EMIT(MCU.ISB);
  (* Round to nearest *)
  SYSTEM.VMSR(0)
END InitFPU;

PROCEDURE Init;
BEGIN
  MCU.Configure();
  (* Map interrupt vectors to SRAM *)
  SYSTEM.PUT(MCU.VTOR, LinkOptions.DataStart);
  InitFPU();
  (* Install SWI Handler *)
  Traps.Init();
  (* Setup oscillator and PLL *)
  SetupPLL();
  (* Initialise UART for runtime error trapping *)
  Serial.Init(Serial.USART2, 38400); 
  Out.Init(Serial.PutCh);
  In.Init(Serial.GetCh)
END Init;

BEGIN
  Init()
END Main.
