MODULE Timers;
(* =========================================================================  
   Astrobe Library Functions for STM32F334 16-bit and 32-bit timers 
   
   (c) 2016 weControl   
   http://www.wecontrol.ch  
   (c) 2012-2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT MCU, SYSTEM;

CONST
  uSecs* = 1000000;
  MSecs* = 1000;

  TIM2* = 2;
  TIM3* = 3;
  TIM6* = 6;
  TIM7* = 7;

  CR1 = 000H;
  EGR = 014H;
  CNT = 024H;
  PSC = 028H;

TYPE
  Timer* = RECORD
    base, multiplier, units: INTEGER
  END;

PROCEDURE* Prescale(VAR timer: Timer);
VAR
  ps, count, divisor: INTEGER;
BEGIN 
  divisor := timer.units;
  count := -1;
  (* Round up instead of truncating.   *)
  (* Max Prescaler is a 16-bit value. *)
  (* Scale by a power of 2 if it is   *)
  (* too large                        *)
  REPEAT
    ps := (MCU.PCLK1 + (divisor DIV 2)) DIV divisor;
    divisor := divisor * 2;
    INC(count)
  UNTIL LSR(ps, 16) = 0;
  timer.multiplier := LSL(1, count);
  SYSTEM.PUT(timer.base + PSC, ps - 1)
END Prescale;


PROCEDURE* Stop*(timer: Timer);
VAR
  cr1: SET;
BEGIN
  SYSTEM.GET(timer.base, cr1);
  SYSTEM.PUT(timer.base, cr1 - {0})
END Stop;


PROCEDURE* Start*(timer: Timer);
VAR
  cr1: SET;
  base: INTEGER;
BEGIN
  base := timer.base;
  (* Stop *)
  SYSTEM.GET(base, cr1);
  SYSTEM.PUT(base, cr1 - {0});
  (* Reset *)
  SYSTEM.PUT(base + EGR, {0});
  (* Enable *)
  SYSTEM.GET(base, cr1);
  SYSTEM.PUT(base, cr1 + {0})
END Start;


PROCEDURE* Elapsed*(timer: Timer): INTEGER;
VAR
  count: INTEGER;
BEGIN
  SYSTEM.GET(timer.base + CNT, count);
  RETURN count DIV timer.multiplier
END Elapsed;


PROCEDURE* Delay*(timer: Timer; delay: INTEGER);
VAR
  count: INTEGER;
  cr1: SET;
  base, cnt: INTEGER;
BEGIN
  IF delay > 0 THEN
    base := timer.base;
    SYSTEM.GET(base, cr1);
    (* Stop *)
    SYSTEM.PUT(base, cr1 - {0});
    (* Reset *)
    SYSTEM.PUT(base + EGR, {0});
    (* Enable *)
    SYSTEM.PUT(base, cr1 + {0});
    delay := delay * timer.multiplier; 
    ASSERT((timer.units = uSecs) OR (LSR(delay, 16) = 0), 22);
    (* Count *)
    cnt := base + CNT;
    REPEAT SYSTEM.GET(cnt, count) UNTIL count >= delay;
    (* Stop *)
    SYSTEM.PUT(base, cr1 - {0})
  ELSE
    ASSERT(delay = 0, 21)
  END
END Delay;


PROCEDURE Init*(VAR timer: Timer; timerNo, units: INTEGER);
VAR
  apb: SET;
BEGIN
  ASSERT((units = MSecs) OR (timerNo = TIM2), 20);
  (* enable peripheral clock for timer *)
  SYSTEM.GET(MCU.RCC_APB1ENR, apb);
  CASE timerNo OF
  | TIM2: 
      timer.base := MCU.TIM2Base; (* CR1 *)
      SYSTEM.PUT(MCU.RCC_APB1ENR, apb + {MCU.RCC_APB1ENR_TIM2})
  | TIM3: 
      timer.base := MCU.TIM3Base; (* CR1 *)
      SYSTEM.PUT(MCU.RCC_APB1ENR, apb + {MCU.RCC_APB1ENR_TIM3})
  | TIM6: 
      timer.base := MCU.TIM6Base; (* CR1 *)
      SYSTEM.PUT(MCU.RCC_APB1ENR, apb + {MCU.RCC_APB1ENR_TIM6})
  | TIM7: 
      timer.base := MCU.TIM7Base; (* CR1 *)
      SYSTEM.PUT(MCU.RCC_APB1ENR, apb + {MCU.RCC_APB1ENR_TIM7})
  END;
  timer.units := units;
  Prescale(timer);
  (* Reset *)
  SYSTEM.PUT(timer.base + EGR, {0})
END Init;

END Timers.
  
